// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/chart/AnyStockLineChartBase",
	"cloudera/Util"
], function(AnyStockLineChartBase, Util) {

return AnyStockLineChartBase.extend({

	/**
	 * A lot of utility methods for manipulating AnyChart's object model.
	 * We have createSomething, which represents a way to create a new
	 * instance of Something.
	 * We have getSomething, which returns Something,
	 * either directly from AnyChart's object model, or from our config object.
	 *
	 * All these methods may be overwritten by child classes.
	 *
	 * AnyChart's JSON data model looks like this:
	 *
	 * data
	 *    dataSets
	 *       one or more dataSet
	 *       each dataSet contains
	 *          a URL
	 *          a id
	 *    dataProviders
	 *       general data providers
	 *           a data provider
	 *              a id
	 *              a dataset id
	 *              a column index.
	 *       scroller data providers
	 *           a data provider
	 *              a id
	 *              a dataset id
	 *              a column index
	 * settings
	 *    charts
	 *       chart
	 *          id
	 *          one or more series
	 *              series
	 *								id
	 *								a data Provider id
	 * annotations
	 *    drawing
	 *
	 * etc.
	 * @see getConfig
	 */
	createData: function(){
		var data = {
			'dataSets': this.createDataSets(),
			'dataProviders': this.createDataProviders()
		};
		return data;
	},

	getData: function() {
		return this.getConfig().data;
	},

	getDataSets: function() {
		return this.getData().dataSets;
	},

	createDataSets: function() {
		return [];
	},

	createDataSet: function(id) {
		var dataset = {
			'dateTimeColumn':0,
			'id': id,
			'locale': {
				'dateTime': {
					'format': '%T',
					'offset': this.getTimezoneOffset()
				}
			},
			'sourceMode':'InternalData',
			'csvSettings': {
				'rowsSeparator': '\n'
			},
			'csvData': ''
		};
		return dataset;
	},

	addDataSet: function(dataSet) {
		return this.addEntry(this.getDataSets(), dataSet);
	},

	getDataSet: function(dataSetId) {
		return this.getEntryById(this.getDataSets(), dataSetId);
	},

	removeDataSet: function(dataSetId) {
		return this.removeEntryById(this.getDataSets(), dataSetId);
	},

	createDataProviders: function() {
		return {
			'generalDataProviders': this.createGeneralDataProviders(),
			'scrollerDataProviders': this.createScrollerDataProviders()
		};
	},

	getDataProviders: function() {
		return this.getData().dataProviders;
	},

	createGeneralDataProviders: function() {
		return [];
	},

	createScrollerDataProviders: function() {
		return [];
	},

	getGeneralDataProviders: function() {
		return this.getDataProviders().generalDataProviders;
	},

	getScrollerDataProviders: function() {
		return this.getDataProviders().scrollerDataProviders;
	},

	createDataProvider: function(dpId, dataSetId, columnIndex) {
		var dp = {
			'id': dpId,
			'dataSet': dataSetId,
			'fields': [{
				'column': columnIndex,
				'type': 'Value'
			}]
		};
		return dp;
	},

	addDataProvider: function(dataProvider) {
		return this.addEntry(this.getGeneralDataProviders(), dataProvider);
	},

	getDataProvider: function(dpId) {
		return this.getEntryById(this.getGeneralDataProviders(), dpId);
	},

	removeDataProvider: function(dpId) {
		return this.removeEntryById(this.getGeneralDataProviders(), dpId);
	},

	createSettings: function() {
		throw new Error("createSettings() should be overwritten");
	},

	getSettings: function() {
		var config = this.getConfig();
		return config.settings;
	},

	createEventMarkers: function() {
		return {};
	},

	getEventMarkers: function() {
		return this.getConfig().eventMarkers;
	},

	createUtils: function() {
		return {};
	},

	getUtils: function() {
		return this.getConfig().utils;
	},

	createAnnotations: function() {
		return {};
	},

	getAnnotations: function() {
		return this.getConfig().annotations;
	},

	createConfig: function() {

		// construct a config object.
		var data = this.createData();
		var settings = this.createSettings();
		var eventMarkers = this.createEventMarkers();
		var utils = this.createUtils();
		var annotations = this.createAnnotations();

		var config = {
			'data': data,
			'settings': settings,
			'eventMarkers': eventMarkers,
			'utils': utils,
			'annotations': annotations
		};
		return config;
	},

	getCharts: function() {
		return this.getSettings().charts;
	},

	addChart: function(chart) {
		return this.addEntry(this.getCharts(), chart);
	},

	getChart: function(chartId) {
		return this.getEntryById(this.getCharts(), chartId);
	},

	removeChart: function(chartId) {
		return this.removeEntryById(this.getCharts(), chartId);
	},

	getSeriesList: function(chartId) {
		var chart = this.getChart(chartId);
		if (chart) {
			return chart.seriesList;
		} else {
			return null;
		}
	},

	createSeriesList: function() {
		return [];
	},

	addSeries: function(chartId, series) {
		return this.addEntry(this.getSeriesList(chartId), series);
	},

	getSeries: function(chartId, seriesId) {
		return this.getEntryById(this.getSeriesList(chartId), seriesId);
	},

	removeSeries: function(chartId, seriesId) {
		return this.removeEntryById(this.getSeriesList(chartId), seriesId);
	},

	/**
	 * Some generic methods to add/remove stuff from arrays.
	 */
	addEntry: function(arr, entry) {
		if (arr) {
			if (arr.push) {
				arr.push(entry);
				return true;
			} else {
				console.log("This is not an array:");
				console.log(arr);
			}
		}
		return false;
	},

	addUniqueEntry: function(arr, entry) {
		var predicate = function(element) {
			return element.id === entry.id;
		};
		var position = this.getPositionByPredicate(arr, predicate);
		if (position === -1) {
			if (this.addEntry(arr, entry)) {
				position = arr.length - 1;
			}
		}
		return position;
	},
	
	moveEntry: function(arr, oldPosition, newPosition) {
		var i, delta;
		if (oldPosition === newPosition) {
			return;
		} else {
			if (oldPosition < newPosition) {
				delta = 1;
			} else {
				delta = -1;
			}
			var temp = arr[oldPosition];
			for (i = oldPosition; i !== newPosition; i += delta) {
				arr[i] = arr[i + delta];
			}
			arr[newPosition] = temp;
		}
	},
	
	getEntry: function(arr, attr, value) {
		var predicate = function(element) {
			return element[attr] === value;
		};
		return this.getEntryByPredicate(arr, predicate);
	},

	getPositionById: function(arr, id) {
		var predicate = function(element) {
			return element.id === id;
		};
		return this.getPositionByPredicate(arr, predicate);
	},

	getPositionByPredicate: function(arr, predicate) {
		var i = 0;
		if (!arr) {
			return -1;
		}

		while(i < arr.length) {
			var element = arr[i];
			if (predicate(element)) {
				return i;
			} else {
				++i;
			}
		}
		return -1;
	},
	
	getEntryByPredicate: function(arr, predicate) {
		var pos = this.getPositionByPredicate(arr, predicate);
		if (pos === -1) {
			return null;
		} else {
			return arr[pos];
		}
	},

	getEntryById: function(arr, id) {
		return this.getEntry(arr, "id", id);
	},

	removeEntry: function(arr, attr, value) {
		var predicate = function(element) {
			return element[attr] === value;
		};
		return this.removeEntryByPredicate(arr, predicate);
	},

	removeEntryByPredicate: function(arr, predicate) {
		var i = 0, result = false;
		while(i < arr.length) {
			if (predicate(arr[i])) {
				arr.splice(i, 1);
				result = true;
			} else {
				++i;
			}
		}
		return result;
	},

	removeEntryById: function(arr, id) {
		return this.removeEntry(arr, "id", id);
	},

	/**
	 * Default implementation of figuring out what start/end time to use.
	 *
	 * We have 6 variables, they could be all different.
	 * first date
	 * first visible date
	 * start date (input)
	 * end date (input)
	 * last visible date
	 * last date
	 */
	initializeTime: function(){
		// console.log("AnyStockLineCharts.initializeTime");

		this.startDate = this.options.startDate;
		this.endDate = this.options.endDate;

		// from the input start/end date
		// how much should we expand to fill out the chart.
		// expandFactor === 1 means
		// startDate should equal firstVisibleDate
		// endDate should equal lastVisibleDate
		var expandFactor = this.getExpandFactor();
		var visibleRange = this.calculateRange(this.startDate, this.endDate, expandFactor);
		this._setVisibleRange(visibleRange);

		// from the visible range
		// how much data (left/right) we should cache.
		// cacheFactor === 2 means
		// we should cache twice as what is visible on the screen.
		var cacheFactor = this.getCacheFactor();
		var cachedRange = this.calculateRange(visibleRange.startDate, visibleRange.endDate, cacheFactor);
		this._setCachedRange(cachedRange);

		this.logTime();
		// console.log("AnyStockLineCharts.initializeTime done");
	},

	onChartDraw: function() {
		this._super.apply(this, arguments);
		this.logTime();
		this.zoomTo(this.firstVisibleDate, this.lastVisibleDate);
	},

	logTime: function(){
		/*
		console.log("        first date:" + this.firstDate);
		console.log("first visible date:" + this.firstVisibleDate);
		console.log(" last visible date:" + this.lastVisibleDate);
		console.log("         last date:" + this.lastDate);
		*/
	},

	/**
	 * The ratio between the specified start/end date
	 * and the visible range of the chart.
	 *
	 * A value of 1 means the input start/end is the range we want to display.
	 */
	getExpandFactor: function() {
		return 1;
	},

	/**
	 * The ratio between the visible range of the chart
	 * and the cached data.
	 *
	 * A value of 1 means no extra caching.
	 */
	getCacheFactor: function() {
		return 1;
	},

	/**
	 * Utility method to compute a new range based on the factor.
	 */
	calculateRange: function(startDate, endDate, factor) {
		var middle = (startDate.getTime() + endDate.getTime()) / 2;
		var delta = endDate.getTime() - startDate.getTime();

		var lastTime = middle + delta * (factor/2);
		var now = Util.getServerNow().getTime();
		if (lastTime > now) {
			lastTime = now;
		}
		return {startDate: new Date(lastTime - delta * factor), endDate: new Date(lastTime)};
	},

	/**
	 * A template method for child classes to override.
	 * Whenever the visible range changes, this method should be called.
	 */
	_checkRange: function() {
	},

	/**
	 * Called when user changes the visible range of the chart.
	 * The default behavior updates the visible range of the chart.
	 * If the phase is 'finish', then call the template method
	 * _checkRange.
	 */
	onSelectedRangeChange: function(chartStartDate, chartEndDate, phase, sender) {

		if (chartStartDate.getTime() !== chartEndDate.getTime()) {
			var visibleRange = {
				startDate: this.fromChartDate(chartStartDate),
				endDate: this.fromChartDate(chartEndDate)
			};

			this._setVisibleRange(visibleRange);

			if (phase === "finish") {
				this._checkRange(false);
			}
		}
	},

	_setVisibleRange: function(range) {
		var result = false;
		if (!isNaN(range.startDate.getTime()) &&
				!isNaN(range.endDate.getTime())) {
			if (!this.firstVisibleDate || this.firstVisibleDate.getTime() !== range.startDate.getTime()) {
				this.firstVisibleDate = range.startDate;
				result = true;
			}
			if (!this.lastVisibleDate || this.lastVisibleDate.getTime() !== range.endDate.getTime()) {
				this.lastVisibleDate = range.endDate;
				result = true;
			}
		}
		return result;
	},

	_setCachedRange: function(range) {
		if (!isNaN(range.startDate.getTime()) &&
				!isNaN(range.endDate.getTime())) {
			this.firstDate = range.startDate;
			this.lastDate = range.endDate;
		}
	},

	onEventMarkerSelect: function(eventMarker) {
		jQuery.publish("eventMarkerSelected", [[eventMarker]]);
	},
	
	onMergedEventMarkerSelect: function(eventMarkers) {
		jQuery.publish("eventMarkerSelected", [eventMarkers]);
	}
});

});
